@@ -1,11 +1,15 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from decimal import Decimal
+
 from trytond import backend
 from trytond.model import ModelView, Workflow, fields
 from trytond.transaction import Transaction
 from trytond.pyson import Eval, If
 from trytond.pool import Pool, PoolMeta
 
+from trytond.modules.product import price_digits
+
 __all__ = ['Configuration', 'ConfigurationSaleMethod', 'Sale', 'SaleLine']
 sale_shipment_cost_method = fields.Selection(
         'get_sale_shipment_cost_methods', "Sale Shipment Cost Method")
@@ -76,7 +80,7 @@ class Sale:
     carrier = fields.Many2One('carrier', 'Carrier',
         domain=[
             ('carrier_product.salable', '=', True),
-            If(Eval('state').in_(['draft', 'quotation']),
+            If(Eval('state') == 'draft',
                 ('id', 'in', Eval('available_carriers', [])),
                 ()),
             ],
@@ -94,6 +98,15 @@ class Sale:
             'readonly': Eval('state') != 'draft',
             }, depends=['state'])
 
+    @classmethod
+    def __setup__(cls):
+        super(Sale, cls).__setup__()
+        cls._error_messages.update({
+                'invalid_carrier': (
+                    "The carrier '%(carrier)s' on sale '%(sale)s' "
+                    "is not valid."),
+                })
+
     @staticmethod
     def default_shipment_cost_method():
         Config = Pool().get('sale.configuration')
@@ -122,13 +135,14 @@ class Sale:
     # XXX We must have the same depends than on_change_with_available_carriers,
     # for now it is maintain manually until we can specify cross-kind depends
     # on on_changes
-    @fields.depends('warehouse', 'shipment_address')
+    @fields.depends('carrier', 'warehouse', 'shipment_address')
     def on_change_party(self):
         super(Sale, self).on_change_party()
         self.available_carriers = self.on_change_with_available_carriers()
-        if self.available_carriers:
+        if self.available_carriers and (not self.carrier
+                or self.carrier not in self.available_carriers):
             self.carrier = self.available_carriers[0]
-        else:
+        elif not self.available_carriers:
             self.carrier = None
 
     @fields.depends('carrier', 'warehouse', 'shipment_address')
@@ -138,7 +152,7 @@ class Sale:
         if self.available_carriers and (not self.carrier
                 or self.carrier not in self.available_carriers):
             self.carrier = self.available_carriers[0]
-        else:
+        elif not self.available_carriers:
             self.carrier = None
 
     @fields.depends('carrier', 'warehouse', 'shipment_address')
@@ -154,7 +168,7 @@ class Sale:
         if self.available_carriers and (not self.carrier
                 or self.carrier not in self.available_carriers):
             self.carrier = self.available_carriers[0]
-        else:
+        elif not self.available_carriers:
             self.carrier = None
 
     @classmethod
@@ -171,6 +185,18 @@ class Sale:
         Line.delete(removed)
         cls.save(sales)
 
+    @classmethod
+    @ModelView.button
+    @Workflow.transition('confirmed')
+    def confirm(cls, sales):
+        for sale in sales:
+            if sale.carrier and sale.carrier not in sale.available_carriers:
+                cls.raise_user_error('invalid_carrier', {
+                        'sale': sale.rec_name,
+                        'carrier': sale.carrier.rec_name,
+                        })
+        super(Sale, cls).confirm(sales)
+
     def _get_carrier_context(self):
         return {}
 
@@ -194,7 +220,7 @@ class Sale:
             date = self.sale_date or today
             with Transaction().set_context(date=date):
                 cost = Currency.compute(Currency(currency_id), cost,
-                    self.currency)
+                    self.currency, round=False)
             cost_line = self.get_shipment_cost_line(cost)
 
         removed = []
@@ -224,6 +250,8 @@ class Sale:
             if last_line.sequence is not None:
                 sequence = last_line.sequence + 1
 
+        shipment_cost = cost.quantize(
+            Decimal(1) / 10 ** SaleLine.shipment_cost.digits[1])
         cost_line = SaleLine(
             sale=self,
             sequence=sequence,
@@ -231,10 +259,12 @@ class Sale:
             product=product,
             quantity=1,  # XXX
             unit=product.sale_uom,
-            shipment_cost=cost,
+            shipment_cost=shipment_cost,
             )
         cost_line.on_change_product()
-        cost_line.unit_price = cost_line.amount = cost
+        cost_line.unit_price = cost.quantize(
+            Decimal(1) / 10 ** SaleLine.unit_price.digits[1])
+        cost_line.amount = cost_line.on_change_with_amount()
         return cost_line
 
     def create_shipment(self, shipment_type):
@@ -247,6 +277,8 @@ class Sale:
                 with Transaction().set_context(
                         shipment.get_carrier_context()):
                     cost, currency_id = self.carrier.get_sale_price()
+                cost = cost.quantize(
+                    Decimal(1) / 10 ** Shipment.cost.digits[1])
                 Shipment.write([shipment], {
                         'carrier': self.carrier.id,
                         'cost': cost,
@@ -281,8 +313,7 @@ class Sale:
 class SaleLine:
     __metaclass__ = PoolMeta
     __name__ = 'sale.line'
-    shipment_cost = fields.Numeric('Shipment Cost',
-        digits=(16, Eval('_parent_sale', {}).get('currency_digits', 2)))
+    shipment_cost = fields.Numeric('Shipment Cost', digits=price_digits)
 
     def _get_invoice_line_quantity(self):
         quantity = super(SaleLine, self)._get_invoice_line_quantity()
@@ -290,3 +321,14 @@ class SaleLine:
                 and self.sale.shipment_cost_method == 'shipment'):
             return 0
         return quantity
+
+
+class Promotion:
+    __metaclass__ = PoolMeta
+    __name__ = 'sale.promotion'
+
+    def get_context_formula(self, sale_line):
+        context = super(Promotion, self).get_context_formula(sale_line)
+        if sale_line.shipment_cost:
+            context['names']['unit_price'] = sale_line.shipment_cost
+        return context
