@@ -1,10 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from decimal import Decimal
+
 from trytond.model import fields
 from trytond.pyson import Eval, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 
+from trytond.modules.product import price_digits
+
 __all__ = ['ShipmentOut']
 
 
@@ -23,15 +27,12 @@ class ShipmentOut:
             'readonly': ~Eval('state').in_(['draft', 'waiting', 'assigned',
                     'packed']),
             }, depends=['carrier', 'state'])
-    cost_currency_digits = fields.Function(fields.Integer(
-        'Cost Currency Digits'),
-        'on_change_with_cost_currency_digits')
     cost = fields.Numeric('Cost',
-        digits=(16, Eval('cost_currency_digits', 2)), states={
+        digits=price_digits, states={
             'invisible': ~Eval('carrier'),
             'readonly': ~Eval('state').in_(['draft', 'waiting', 'assigned',
                     'packed']),
-            }, depends=['carrier', 'state', 'cost_currency_digits'])
+            }, depends=['carrier', 'state'])
     cost_invoice_line = fields.Many2One('account.invoice.line',
             'Cost Invoice Line', readonly=True)
 
@@ -43,12 +44,6 @@ class ShipmentOut:
                     'product "%s".'),
                 })
 
-    @fields.depends('cost_currency')
-    def on_change_with_cost_currency_digits(self, name=None):
-        if self.cost_currency:
-            return self.cost_currency.digits
-        return 2
-
     def _get_carrier_context(self):
         return {}
 
@@ -69,10 +64,9 @@ class ShipmentOut:
             return
         with Transaction().set_context(self._get_carrier_context()):
             cost, currency_id = self.carrier.get_sale_price()
-        self.cost = cost
+        self.cost = cost.quantize(
+            Decimal(1) / 10 ** self.__class__.cost.digits[1])
         self.cost_currency = currency_id
-        self.cost_currency_digits = (self.cost_currency.digits
-            if self.cost_currency else 2)
 
     def _get_cost_tax_rule_pattern(self):
         'Get tax rule pattern for invoice line'
@@ -104,8 +98,9 @@ class ShipmentOut:
         if invoice.currency != self.cost_currency:
             with Transaction().set_context(date=invoice.currency_date):
                 cost = Currency.compute(self.cost_currency, cost,
-                    invoice.currency)
-        invoice_line.unit_price = cost
+                    invoice.currency, round=False)
+        invoice_line.unit_price = cost.quantize(
+            Decimal(1) / 10 ** InvoiceLine.unit_price.digits[1])
 
         taxes = []
         pattern = self._get_cost_tax_rule_pattern()
